<?php
namespace {

    defined('ABSPATH') or exit;
}

namespace Baikalsr {
	
	use Baikalsr\ShippingMethod;
	use Baikalsr\Fields;
	use Baikalsr\Tools;
	use WC_Shipping_Method;
	use WC_Order_Item_Shipping;
	
	class Plugin {
		
		public const METHOD_ID = 'baikalsr_shipping';	
		public $shippingMethod;
		
		public function __construct() {
			// apply plugin textdomain.
			add_action( 'plugins_loaded', 
				static fn() => load_plugin_textdomain('baikalsr-shipping', false, dirname(BS_PLUGIN_BASENAME).'/lang'));
			add_action( 'wp_enqueue_scripts', array( $this, 'frontend_enqueue_scripts' ) );
			add_action( 'admin_enqueue_scripts', array( $this, 'backend_enqueue_scripts') );
			//Declare HPOS compatibility
			add_action('before_woocommerce_init', function() {
				if( !class_exists( \Automattic\WooCommerce\Utilities\FeaturesUtil::class ) ) {
					return;
				}
				\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility('custom_order_tables', BS_PLUGIN_FILE);
			});
			add_action('woocommerce_shipping_init', array( $this, 'init_shipping' ));
			add_action('woocommerce_shipping_methods',
					static fn($methods) => array_merge($methods, [self::METHOD_ID => '\Baikalsr\ShippingMethod']));
					
			add_filter('woocommerce_checkout_create_order_shipping_item', array( $this, 'checkout_field_update_order_meta') );
			add_action('woocommerce_before_order_itemmeta', array( $this, 'admin_before_order_itemmeta'), 10, 2);
			add_filter('woocommerce_hidden_order_itemmeta', array( $this, 'admin_shipping_hidden_meta'));
			add_action('woocommerce_after_order_itemmeta', array( $this, 'admin_after_order_itemmeta'), 10, 2);
			
			add_action('woocommerce_after_shipping_rate', array( $this, 'display_description_if_present' ));
			
			add_filter('plugin_action_links_' . BS_PLUGIN_BASENAME, array( $this, 'plugin_action_links' ) );
			//AJAX
			add_action( 'wp_ajax_bs_callback', array( $this, 'plgWcOnSelfCallBE' ) );

		}
		
		/**
		* Save field values
		*/
		public function checkout_field_update_order_meta( $shipping ) {
			if ($shipping->get_method_id() !== self::METHOD_ID) {
                return;
            }
			if( $shipping->get_meta('_smethod') == 'pickup' ){
				if(isset( $_POST['bs_terminal_id'] ) ) {
					$bs_terminal_id = !empty($_POST['bs_terminal_id']) ? $_POST['bs_terminal_id'] : 0;
					$terminals = $shipping->get_meta('_terminals');
					foreach($terminals as $terminal){
						if(isset($terminal['id']) && $terminal['id'] ==  $bs_terminal_id){
							$shipping->add_meta_data('_terminal_id', sanitize_text_field( $terminal['id']));
							if(isset($terminal['name']) && !empty($terminal['name'])){
								$shipping->add_meta_data('_terminal_name', sanitize_text_field( $terminal['name']));
							}
							if(isset($terminal['address']) && !empty($terminal['address'])){
								$shipping->add_meta_data('_terminal_address', sanitize_text_field( $terminal['address']));
							}
							$shipping->save_meta_data();
							break;
						}
					}
				}
			}
		}
		
		/**
		* Display field value on the order edit page
		*/
		public function admin_before_order_itemmeta( $item_id, $item ){
			if (!$item instanceof WC_Order_Item_Shipping || $item->get_method_id() !== self::METHOD_ID) {
				return;
			}
			$description = '';
			foreach ($item->get_meta_data() as $value) {
				
				$meta = $value->get_data();
				if($meta['key'] == '_terminal_name'){
					$description .= esc_html__('Terminal', 'baikalsr-shipping') . '&nbsp;«' . esc_attr( $meta['value']) . '»';
				}elseif($meta['key'] == '_terminal_address'){
					$description .= '&nbsp;(' . esc_attr( $meta['value']) . ')';
				}
			}
			if(!empty($description))
				echo '<div>' . $description . '</div>';
		}
		
		/**
		* Display field value on the order edit page
		*/
		public function admin_after_order_itemmeta( $item_id, $item ){
			if (!$item instanceof WC_Order_Item_Shipping || $item->get_method_id() !== self::METHOD_ID) {
				return;
			}
			$order = wc_get_order($item->get_order_id());
			if(!empty($order)){
				wc_get_template( 'order/bs-popup-button.php', $this->getShippingInfoData($item), '', BS_PLUGIN_DIR_PATH . '/templates/' );	
				wc_get_template( 'order/bs-popup-shipping.php', $this->getShippingPopupData($order), '', BS_PLUGIN_DIR_PATH . '/templates/' );
			}
		}
		
		public function getShippingInfoData($shipping)
		{
			$template_args = [];
			$template_args['order_id'] = $shipping->get_order_id();
			foreach ($shipping->get_meta_data() as $value) {		
				$meta = $value->get_data();
				if($meta['key'] == '_request_id'){
					$template_args['request_id'] =  esc_attr( $meta['value']);
				}elseif($meta['key'] == '_status_name'){
					$template_args['status_name'] =  esc_attr( $meta['value']);
				}elseif($meta['key'] == '_actual_cost'){
					$template_args['actual_cost'] =  esc_attr( $meta['value']);
				}	
			}
			return $template_args;
		}
		
		public function getShippingPopupData($order, $post_data = array()){
			
			$config = WC()->shipping->load_shipping_methods()[self::METHOD_ID];
			if(empty($config)){
				return; //sprintf(esc_html__('No Config Found %s', 'baikalsr-shipping'), $order_id);
			}
			foreach ( $order->get_shipping_methods() as $order_item_shipping ) {
				if ( $order_item_shipping instanceof WC_Order_Item_Shipping && $order_item_shipping->get_method_id() == self::METHOD_ID ) 
					$shipping = $order_item_shipping;
			}
			if(empty($shipping)){
				return; //sprintf(esc_html__('No Config Found %s', 'baikalsr-shipping'), $order_id);
			}
			$form_data = [];
			$_shipment_data = [];
			foreach ($shipping->get_meta_data() as $value) {		
				$meta = $value->get_data();
				if($meta['key'] == '_shipment_data'){
					$_shipment_data = $meta['value'];					
				}
			}
			
			$form_data['popup_style'] = 'display:none;';
			$form_data['disabled'] = false;	
			if(!empty($_shipment_data)){
				$form_data['disabled'] = true;			
				$sent_data = $_shipment_data;
			}elseif(!empty($post_data)){
				$form_data['popup_style'] = '';
				$sent_data = $post_data;
				$form_data['warnings'] = $this->validateForm($sent_data);
				if(count($form_data['warnings']) == 0){
					$baikalResponse = Tools::order($sent_data);
					$form_data['errors'] = [];
					if(empty($baikalResponse)) {
						$form_data['errors']['no_response'] = esc_html__('No Response', 'baikalsr-shipping');
					}elseif(isset($baikalResponse["errors"]) && !empty($baikalResponse["errors"])) {
						//$form_data['errors']['invalid_data'] = implode("</li><li>", $baikalResponse["errors"]);
						foreach($baikalResponse["errors"] as $errorData)
							$form_data['errors']['invalid_data'] .= urldecode(str_replace("=", ": ", http_build_query($errorData,"", "\r\n")));							
					}elseif(!isset($baikalResponse["request_hash"]) || empty($baikalResponse["request_hash"])){
						$form_data['errors']['no_hash'] = esc_html__('The hash is missing in the server response', 'baikalsr-shipping');
					}elseif(!isset($baikalResponse["request_id"]) || empty($baikalResponse["request_id"])){
						$form_data['errors']['no_request'] = esc_html__('The request number is missing in the server response', 'baikalsr-shipping');
					}elseif(!isset($baikalResponse["tracknumbers"]) || empty($baikalResponse["tracknumbers"])){
						$form_data['errors']['no_tracking'] = esc_html__('The tracking number is missing in the server response', 'baikalsr-shipping');
					}
					if(count($form_data['errors']) == 0){
						$shipping->add_meta_data('_request_id', sanitize_text_field( $baikalResponse["request_id"]));
						$shipping->add_meta_data('_tracking_number', sanitize_text_field( $baikalResponse["tracknumbers"][0]));
						$shipping->add_meta_data('_status_name', sanitize_text_field(esc_html__('Not processed', 'baikalsr-shipping'))); 
						$shipping->add_meta_data('_actual_cost', $shipping->get_total()); 
						$shipping->add_meta_data('_shipment_data', $sent_data);
						$shipping->save_meta_data();
						$form_data['disabled'] = true;	
						$form_data['success'] = true;					
					}
				}
			}
			foreach ($shipping->get_meta_data() as $value) {		
				$meta = $value->get_data();
				if($meta['key'] == '_tracking_number'){
					$form_data['tracking_number'] =  esc_attr( $meta['value']);
				}elseif($meta['key'] == '_request_id'){
					$form_data['request_id'] =  esc_attr( $meta['value']);
				}
			}
			//Returns an escaped string
			foreach ($sent_data as $key=>$value) {
				$sent_data[$key] = stripcslashes($value);
			}
			$bsFields = new Fields();
			$form_data['order_id'] = $shipping->get_order_id();
			//Consignor
			if (isset($sent_data['consignor_contragent_id'])) {
				$form_data['consignor_contragent_id'] = $sent_data['consignor_contragent_id'];
			} else {
				$form_data['consignor_contragent_id'] = $config->get_option( 'consignor_contragent_id' );
			}	
			if (isset($sent_data['consignor_contragents'])) {
				$form_data['consignor_contragents'] = $sent_data['consignor_contragents'];
			} else {
				$form_data['consignor_contragents'] = $bsFields->getContragents();
			}	
			if (isset($sent_data['author_lastname'])) {
				$form_data['author_lastname'] = $sent_data['author_lastname'];
			} else {
				$form_data['author_lastname'] = $config->get_option( 'author_lastname' );
			}	
			if (isset($sent_data['author_firstname'])) {
				$form_data['author_firstname'] = $sent_data['author_firstname'];
			} else {
				$form_data['author_firstname'] = $config->get_option( 'author_firstname');
			}	
			if (isset($sent_data['author_middlename'])) {
				$form_data['author_middlename'] = $sent_data['author_middlename'];
			} else {
				$form_data['author_middlename'] = $config->get_option( 'author_middlename');
			}
			if (isset($sent_data['consignor_fio'])) {
				$form_data['consignor_fio'] = $sent_data['consignor_fio'];
			} else {
				$form_data['consignor_fio'] = $form_data['author_lastname'] . ' '. $form_data['author_firstname'] . ' ' . $form_data['author_middlename'];
			}						
			if (isset($sent_data['consignor_phone'])) {
				$form_data['consignor_phone'] = $sent_data['consignor_phone'];
			} else {
				$form_data['consignor_phone'] = $config->get_option('consignor_phone');
			}		
			if (isset($sent_data['consignor_email'])) {
				$form_data['consignor_email'] = $sent_data['consignor_email'];
			} else {
				$form_data['consignor_email'] = $config->get_option('consignor_email');
			}
			//Consignee
			if (isset($sent_data['consignee_company'])) {
				$form_data['consignee_company'] = $sent_data['consignee_company'];
			} else {
				$form_data['consignee_company'] = $order->get_shipping_company() ?: $order-> get_billing_company();			
			}
			$form_data['consignee_company'] = htmlspecialchars_decode($form_data['consignee_company']);
			if (isset($sent_data['consignee_physical_person'])) {
				$form_data['consignee_physical_person'] = $sent_data['consignee_physical_person'];
			} else {
				if (isset($form_data['consignee_company']) && !empty($form_data['consignee_company'])) {
					$form_data['consignee_physical_person'] = 0;
				}else{
					$form_data['consignee_physical_person'] = 1;
				}
			}
			if (isset($sent_data['consignee_inn'])) {
				$form_data['consignee_inn'] = $sent_data['consignee_inn'];
			} else {
				$form_data['consignee_inn'] = '';		
			}	
			if (isset($sent_data['consignee_kpp'])) {
				$form_data['consignee_kpp'] = $sent_data['consignee_kpp'];
			} else {
				$form_data['consignee_kpp'] = '';
			}
			if (isset($sent_data['consignee_inn_kpp'])) {
				$form_data['consignee_inn_kpp'] = $sent_data['consignee_inn_kpp'];
			} else {
				$form_data['consignee_inn_kpp'] = trim($form_data['consignee_inn'] . ' ' . $form_data['consignee_kpp']);
				$pos = strpos($form_data['consignee_inn_kpp'], ' ');
				if ($pos !== false) {
					$form_data['consignee_inn_kpp'] = substr_replace($form_data['consignee_inn_kpp'], '/', $pos, strlen(' '));
				}
			}	
			if (isset($sent_data['consignee_fio'])) {
				$form_data['consignee_fio'] = $sent_data['consignee_fio'];
			} else {
				$form_data['consignee_fio'] = trim(($order->get_shipping_last_name() ?: $order->get_billing_last_name()) .
                        ' ' . ($order->get_shipping_first_name() ?: $order->get_billing_first_name()));	
			}
			$form_data['legal_opf'] = $bsFields->getOpf();
			if (isset($sent_data['consignee_legal_opf_id'])) {
				$form_data['consignee_legal_opf_id'] = $sent_data['consignee_legal_opf_id'];
			} else {
				$form_data['consignee_legal_opf_id'] = 4;
			}	
			$form_data['physical_documents'] = $bsFields->getDocuments();
			if (isset($sent_data['consignee_physical_document_id'])) {
				$form_data['consignee_physical_document_id'] = $sent_data['consignee_physical_document_id'];
			} else {
				$form_data['consignee_physical_document_id'] = 1;
			}
			if (!isset($sent_data['consignee_no_document'])) {
				$form_data['consignee_no_document'] = false;
			} else {
				$form_data['consignee_no_document'] = $sent_data['consignee_no_document'];
			}
			if (isset($sent_data['consignee_seria'])) {
				$form_data['consignee_seria'] = $sent_data['consignee_seria'];
			} else {
				$form_data['consignee_seria'] = '';	
			}	
			if (isset($sent_data['consignee_number'])) {
				$form_data['consignee_number'] = $sent_data['consignee_number'];
			} else {
				$form_data['consignee_number'] = '';
			}
			if (isset($sent_data['consignee_seria_number'])) {
				$form_data['consignee_seria_number'] = $sent_data['consignee_seria_number'];
			} else {
				$form_data['consignee_seria_number'] = trim($form_data['consignee_seria'] . ' ' . $form_data['consignee_number']);
			}							
			if (isset($sent_data['consignee_phone'])) {
				$form_data['consignee_phone'] = $sent_data['consignee_phone'];
			} else {
				$form_data['consignee_phone'] = trim($order->get_shipping_phone() ?: $order->get_billing_phone());				
			}					
			if (isset($sent_data['consignee_email'])) {
				$form_data['consignee_email'] = $sent_data['consignee_email'];
			} else {
				$form_data['consignee_email'] = $order->get_billing_email();
			}	
			if (isset($sent_data['departure_guid'])) {
				$form_data['departure_guid'] = $sent_data['departure_guid'];
			} else {
				$form_data['departure_guid'] = $config->get_option('departure_guid');
			}
			if (isset($sent_data['departure'])) {
				$form_data['departure'] = $sent_data['departure'];
			} else {
				$form_data['departure'] = $config->get_option('departure');
			}	
			if (isset($sent_data['departure_from_address'])) {
				$form_data['departure_from_address'] = $sent_data['departure_from_address'];
			} else {
				$form_data['departure_from_address'] = $config->get_option('departure_from_address');
			}
			if (isset($sent_data['departure_date'])) {
				$form_data['departure_date'] = $sent_data['departure_date'];
			} else {
				if($form_data['departure_from_address'])
					$form_data['departure_date'] = date("Y-m-d", strtotime("+1 day"));
				else			
					$form_data['departure_date'] = date("Y-m-d");
			}		
			if (isset($sent_data['departure_terminal_id'])) {
				$form_data['departure_terminal_id'] = $sent_data['departure_terminal_id'];
			} else {
				$form_data['departure_terminal_id'] = $config->get_option('departure_terminal_id');
			}	
			$form_data['departure_terminals'] = $bsFields->getTerminals($form_data['departure_guid'], "start");
			if (isset($sent_data['departure_street_guid'])) {
				$form_data['departure_street_guid'] = $sent_data['departure_street_guid'];
			} else {
				$form_data['departure_street_guid'] = $config->get_option('departure_street_guid');
			}
			if (isset($sent_data['departure_street'])) {
				$form_data['departure_street'] = $sent_data['departure_street'];
			} else {
				$form_data['departure_street'] = $config->get_option('departure_street');
			}
			if (isset($sent_data['departure_house_guid'])) {
				$form_data['departure_house_guid'] = $sent_data['departure_house_guid'];
			} else {
				$form_data['departure_house_guid'] = $config->get_option('departure_house_guid');
			}
			if (isset($sent_data['departure_house'])) {
				$form_data['departure_house'] = $sent_data['departure_house'];
			} else {
				$form_data['departure_house'] = $config->get_option('departure_house');
			}
			if (isset($sent_data['departure_housing'])) {
				$form_data['departure_housing'] = $sent_data['departure_housing'];
			} else {
				$form_data['departure_housing'] = $config->get_option('departure_housing');
			}
			if (isset($sent_data['departure_building'])) {
				$form_data['departure_building'] = $sent_data['departure_building'];
			} else {
				$form_data['departure_building'] = $config->get_option('departure_building');
			}
			if (isset($sent_data['departure_apartment'])) {
				$form_data['departure_apartment'] = $sent_data['departure_apartment'];
			} else {
				$form_data['departure_apartment'] = $config->get_option('departure_apartment');
			}
			if (isset($sent_data['departure_comment'])) {
				$form_data['departure_comment'] = $sent_data['departure_comment'];
			} else {
				$form_data['departure_comment'] = $config->get_option('departure_comment');
			}
			if (isset($sent_data['departure_prr'])) {
				$form_data['departure_prr'] = $sent_data['departure_prr'];
			} else {
				$form_data['departure_prr'] = $config->get_option('departure_prr') == 'yes'? 1 : 0;
			}
			if (isset($sent_data['departure_floor'])) {
				$form_data['departure_floor'] = $sent_data['departure_floor'];
			} else {
				$form_data['departure_floor'] = $config->get_option('departure_floor');
				if(empty($form_data['departure_floor'])) 
					$form_data['departure_floor'] = 1;
			}
			if (isset($sent_data['departure_lift'])) {
				$form_data['departure_lift'] = $sent_data['departure_lift'];
			} else {
				$form_data['departure_lift'] = $config->get_option('departure_lift') == 'yes'? 1 : 0;
			}	
			if (isset($sent_data['destination_guid'])) {
				$form_data['destination_guid'] = $sent_data['destination_guid'];
			} else {
				$form_data['destination_guid'] = '';
				foreach ($shipping->get_meta_data() as $value) {		
					$meta = $value->get_data();
					if($meta['key'] == '_destination_guid'){
						$form_data['destination_guid'] = esc_attr( $meta['value']);
					}
				}
			}
			if (isset($sent_data['destination'])) {
				$form_data['destination'] = $sent_data['destination'];
			} else {
				$form_data['destination'] = '';
				foreach ($shipping->get_meta_data() as $value) {		
					$meta = $value->get_data();
					if($meta['key'] == '_destination_name'){
						$form_data['destination'] = esc_attr( $meta['value']);
					}
				}
			}
			if (isset($sent_data['destination_to_address'])) {
				$form_data['destination_to_address'] = $sent_data['destination_to_address'];
			} else {
				$form_data['destination_to_address'] = 0;
				$selected = null;
				foreach ($shipping->get_meta_data() as $value) {		
					$meta = $value->get_data();
					if($meta['key'] == '_smethod'){
						$selected = esc_attr( $meta['value']);
					}
				}
				if ($selected) {
					if ($selected=='courier')
						$form_data['destination_to_address'] = 1;
					elseif ($selected=='pickup')
						$form_data['destination_to_address'] = 0;
				}				
			}
			if (isset($sent_data['destination_date'])) {
				$form_data['destination_date'] = $sent_data['destination_date'];
			} else {
				if($form_data['departure_from_address'] && !$form_data['destination_to_address'])
					$form_data['destination_date'] = date("Y-m-d", strtotime("+2 day"));
				elseif($form_data['departure_from_address'] && $form_data['destination_to_address'])
					$form_data['destination_date'] = date("Y-m-d", strtotime("+3 day"));
				elseif(!$form_data['departure_from_address'] && $form_data['destination_to_address'])
					$form_data['destination_date'] = date("Y-m-d", strtotime("+2 day"));
				else
					$form_data['destination_date'] = date("Y-m-d", strtotime("+1 day"));
			}		
			$form_data['destination_terminals'] = $bsFields->getTerminals($form_data['destination_guid'], "finish");			
			if (isset($sent_data['destination_terminal_id'])) {
				$form_data['destination_terminal_id'] = $sent_data['destination_terminal_id'];
			} else {
				$form_data['destination_terminal_id'] = -1;
				$selected = null;
				foreach ($shipping->get_meta_data() as $value) {		
					$meta = $value->get_data();
					if($meta['key'] == '_terminal_id'){
						$selected = esc_attr( $meta['value']);
					}
				}
				if ($selected) {
					$form_data['destination_terminal_id'] = $selected;
				}		
			}
			if (isset($sent_data['destination_street_guid'])) {
				$form_data['destination_street_guid'] = $sent_data['destination_street_guid'];
			} else {
				$form_data['destination_street_guid'] = '';
			}
			if (isset($sent_data['destination_street'])) {
				$form_data['destination_street'] = $sent_data['destination_street'];
			} else {
				$form_data['destination_street'] = '';//$order->get_shipping_address_1() . ', ' . $order->get_shipping_address_2();
			}	
			if (isset($sent_data['destination_house_guid'])) {
				$form_data['destination_house_guid'] = $sent_data['destination_house_guid'];
			} else {
				$form_data['destination_house_guid'] = '';
			}
			if (isset($sent_data['destination_house'])) {
				$form_data['destination_house'] = $sent_data['destination_house'];
			} else {
				$form_data['destination_house'] = '';
			}
			if (isset($sent_data['destination_housing'])) {
				$form_data['destination_housing'] = $sent_data['destination_housing'];
			} else {
				$form_data['destination_housing'] = '';
			}
			if (isset($sent_data['destination_building'])) {
				$form_data['destination_building'] = $sent_data['destination_building'];
			} else {
				$form_data['destination_building'] = '';
			}
			if (isset($sent_data['destination_apartment'])) {
				$form_data['destination_apartment'] = $sent_data['destination_apartment'];
			} else {
				$form_data['destination_apartment'] = '';
			}	
			if (isset($sent_data['destination_comment'])) {
				$form_data['destination_comment'] = $sent_data['destination_comment'];
			} else {
				$form_data['destination_comment'] = '';
				if(!empty($order->get_billing_state()))
					$form_data['destination_comment'] = $order->get_billing_state();
				if(!empty($order->get_shipping_state()))
					$form_data['destination_comment'] = $order->get_shipping_state();	
				$form_data['destination_comment'] .= ' ' . $form_data['destination'] . ' ';
				if(!$form_data['destination_pickup']){
					if(!empty($order->get_shipping_address_1())){
						$form_data['destination_comment'] .=  trim($order->get_shipping_address_1() . ' ' . $order->get_shipping_address_2());
					}elseif(!empty($order->get_billing_address_1())){
						$form_data['destination_comment'] .=  trim($order->get_billing_address_1() . ' ' . $order->get_billing_address_2());
					}
				}else{
					$selected = null;
					foreach ($shipping->get_meta_data() as $value) {		
						$meta = $value->get_data();
						if($meta['key'] == '_terminal_name'){
							$selected = esc_attr( $meta['value']);
						}
					}
					if ($selected) 
						$form_data['destination_comment'] .= '. ' . esc_html__('Terminal Address', 'baikalsr-shipping'). ': ' . $selected;
				}
				$form_data['destination_comment'] = trim($form_data['destination_comment']);
			}	
			if (isset($sent_data['destination_prr'])) {
				$form_data['destination_prr'] = $sent_data['destination_prr'];
			} else {
				$form_data['destination_prr'] = 0;
			}	
			if (isset($sent_data['destination_floor'])) {
				$form_data['destination_floor'] = $sent_data['destination_floor'];
			} else {
				$form_data['destination_floor'] = 1;
			}
			if (isset($sent_data['destination_lift'])) {
				$form_data['destination_lift'] = $sent_data['destination_lift'];
			} else {
				$form_data['destination_lift'] = 1;
			}	
			$form_data['cargo_types'] = $bsFields->getCargoTypes();
			if (isset($sent_data['cargotype_id'])) {
				$form_data['cargotype_id'] = $sent_data['cargotype_id'];
			} else {
				$form_data['cargotype_id'] = $config->get_option('cargotype_id');
			}	
			if (isset($sent_data['packing_id'])) {
				$form_data['packing_id'] = $sent_data['packing_id'];
			} else {
				$form_data['packing_id'] = $config->get_option('packing_id');
			}		
			$form_data['packing'] = $bsFields->getPacking();						
			$prodDimensions = $this->shippingMethod->_getDimensionsAndWeight($order->get_items(), $this->shippingMethod->settings);
			if (isset($sent_data['volume'])) {
				$form_data['volume'] = $sent_data['volume'];
			} else {
				$form_data['volume'] = $prodDimensions['volume'];
			}	
			if (isset($sent_data['weight'])) {
				$form_data['weight'] = $sent_data['weight'];
			} else {
				$form_data['weight'] = $prodDimensions['weight'];
			}	
			if (isset($sent_data['length'])) {
				$form_data['length'] = $sent_data['length'];
			} else {
				$form_data['length'] = $prodDimensions['length'];
			}	
			if (isset($sent_data['width'])) {
				$form_data['width'] = $sent_data['width'];
			} else {
				$form_data['width'] = $prodDimensions['width'];
			}	
			if (isset($sent_data['height'])) {
				$form_data['height'] = $sent_data['height'];
			} else {
				$form_data['height'] = $prodDimensions['height'];
			}	
			if (isset($sent_data['estimated_cost'])) {
				$form_data['estimated_cost'] = $sent_data['estimated_cost'];
			} else {
				$form_data['estimated_cost'] = 0;
				foreach ( $order->get_items() as $item_id => $item ) {
					$form_data['estimated_cost'] += round($item->get_subtotal() + $item->get_subtotal_tax(), 2);
				}
			}			
			return $form_data;
		}	
		
		public function validateForm($form_data) {

			$warning = [];
			if(!isset($form_data['consignor_contragent_id']) || $form_data['consignor_contragent_id'] == ''
				|| $form_data['consignor_contragent_id'] == '-1')
				$warning['consignor_contragent'] = esc_html__('The counterparty is not specified', 'baikalsr-shipping');
			if(!isset($form_data['consignor_fio']) || $form_data['consignor_fio'] == '')
				$warning['consignor_fio'] = esc_html__('Full name not specified', 'baikalsr-shipping');
			if(!isset($form_data['author_firstname']) || $form_data['author_firstname'] == '')
				$warning['consignor_fio'] = esc_html__('Full name not specified', 'baikalsr-shipping');
			if(!isset($form_data['author_middlename']) || $form_data['author_middlename'] == '')
				$warning['consignor_fio'] = esc_html__('Full name not specified', 'baikalsr-shipping');
			if(!isset($form_data['author_lastname']) || $form_data['author_lastname'] == '')
				$warning['consignor_fio'] = esc_html__('Full name not specified', 'baikalsr-shipping');
			if(!isset($form_data['consignor_phone']) || $form_data['consignor_phone'] == '')
				$warning['consignor_phone'] = esc_html__('Phone number not specified', 'baikalsr-shipping');
			elseif(strlen(preg_replace('![^0-9]+!', '', $form_data['consignor_phone'])) < 11)
				$warning['consignor_phone'] = esc_html__('The phone number cannot be less than 11 digits long', 'baikalsr-shipping');
			if(!isset($form_data['consignor_email']) || $form_data['consignor_email'] == '')
				$warning['consignor_email'] = esc_html__('Email not specified', 'baikalsr-shipping');
			if(isset($form_data['consignee_physical_person']) && $form_data['consignee_physical_person'] != '1'){
				if(!isset($form_data['consignee_company']) || $form_data['consignee_company'] == '')
					$warning['consignee_company'] = esc_html__('The company is not specified', 'baikalsr-shipping');
				if(!isset($form_data['consignee_legal_opf_id']) || $form_data['consignee_legal_opf_id'] == '' 
					|| $form_data['consignee_legal_opf_id'] == '-1')
					$warning['consignee_legal_opf'] = esc_html__('The legal form is not selected', 'baikalsr-shipping');
				if(!isset($form_data['consignee_inn_kpp']) || $form_data['consignee_inn_kpp'] == ''){
					$warning['consignee_inn_kpp'] = esc_html__('TIN is not specified', 'baikalsr-shipping');	
				}elseif(isset($form_data['consignee_legal_opf_id']) && !empty($form_data['consignee_legal_opf_id']) 
				&& $form_data['consignee_legal_opf_id'] != '9'){
					if( strlen(preg_replace('/[^0-9.]/', '', $form_data['consignee_inn']))<10)
						$warning['consignee_inn_kpp'] = sprintf(esc_html__('The specified INN is less than %s characters', 'baikalsr-shipping'), '10');	
					if( strlen(preg_replace('/[^0-9.]/', '', $form_data['consignee_kpp']))<9)
						$warning['consignee_inn_kpp'] = sprintf(esc_html__('The specified IC is less than %s characters', 'baikalsr-shipping'), '9');					
				}elseif(isset($form_data['consignee_legal_opf_id']) && !empty($form_data['consignee_legal_opf_id']) 
				&& $form_data['consignee_legal_opf_id'] == '9' && strlen(preg_replace('/[^0-9.]/', '', $form_data['consignee_inn']))<12){
					$warning['consignee_inn_kpp'] = sprintf(esc_html__('The specified INN is less than %s characters', 'baikalsr-shipping'), '12');	
				}
			}
			if(!isset($form_data['consignee_fio']) || $form_data['consignee_fio'] == '')
				$warning['consignee_fio'] = esc_html__('Full name not specified', 'baikalsr-shipping');			
			if(isset($form_data['consignee_physical_person']) && $form_data['consignee_physical_person'] == '1'){
				if(!isset($form_data['consignee_no_document']) || $form_data['consignee_no_document'] != '1'){
					if(!isset($form_data['consignee_physical_document_id']) || $form_data['consignee_physical_document_id'] == '' 
						|| $form_data['consignee_physical_document_id'] == '-1')
						$warning['consignee_physical_document'] = esc_html__('The document is not selected', 'baikalsr-shipping');	
					if(!isset($form_data['consignee_seria_number']) || $form_data['consignee_seria_number'] == '')
						$warning['consignee_seria_number'] = esc_html__('The series/number is not specified', 'baikalsr-shipping');		
				}
			}
			if(!isset($form_data['consignee_phone']) || $form_data['consignee_phone'] == '')
				$warning['consignee_phone'] = esc_html__('Phone number not specified', 'baikalsr-shipping');
			elseif(strlen(preg_replace('![^0-9]+!', '', $form_data['consignee_phone'])) < 11)
				$warning['consignee_phone'] = esc_html__('The phone number cannot be less than 11 digits long', 'baikalsr-shipping');
			if(!isset($form_data['consignee_email']) || $form_data['consignee_email'] == '')
				$warning['consignee_email'] = esc_html__('Email not specified', 'baikalsr-shipping');
			if(!isset($form_data['departure_guid']) || $form_data['departure_guid'] == '')
				$warning['departure'] = esc_html__('The departure point is not selected', 'baikalsr-shipping');
			if(!isset($form_data['departure']) || $form_data['departure'] == '')
				$warning['departure'] = esc_html__('The departure point is not selected', 'baikalsr-shipping');
			if(isset($form_data['departure_from_address']) && $form_data['departure_from_address'] != '1'){
				if(!isset($form_data['departure_terminal_id']) || $form_data['departure_terminal_id'] == '' 
					|| $form_data['departure_terminal_id'] == '-1')
					$warning['departure_terminal_id'] = esc_html__('The terminal is not selected', 'baikalsr-shipping');					
			}	
			if(isset($form_data['departure_from_address']) && $form_data['departure_from_address'] == '1'){
				if(!isset($form_data['departure_street_guid']) || $form_data['departure_street_guid'] == '')
					$warning['departure_street'] = esc_html__('The street is not specified', 'baikalsr-shipping');		
				if(!isset($form_data['departure_street']) || $form_data['departure_street'] == '')
					$warning['departure_street'] = esc_html__('The street is not specified', 'baikalsr-shipping');		
				//Можно вводить дом без классификатора
				//if(!isset($form_data['departure_house_guid']) || $form_data['departure_house_guid'] == '')
				//	$warning['departure_house'] = esc_html__('The house is not specified', 'baikalsr-shipping');		
				if(!isset($form_data['departure_house']) || $form_data['departure_house'] == '')
					$warning['departure_house'] = esc_html__('The house is not specified', 'baikalsr-shipping');	
				if(isset($form_data['departure_prr']) && $form_data['departure_prr'] == '1'){
					if(!isset($form_data['departure_floor']) || $form_data['departure_floor'] == '')
						$warning['departure_floor'] = esc_html__('Floor not specified', 'baikalsr-shipping');	
				}
			}	
			if(!isset($form_data['destination_guid']) || $form_data['destination_guid'] == '')
				$warning['destination'] = esc_html__('The Destination point is not selected', 'baikalsr-shipping');
			if(!isset($form_data['destination']) || $form_data['destination'] == '')
				$warning['destination'] = esc_html__('The Destination point is not selected', 'baikalsr-shipping');
			if(isset($form_data['destination_to_address']) && $form_data['destination_to_address'] != '1'){
				if(!isset($form_data['destination_terminal_id']) || $form_data['destination_terminal_id'] == '' 
					|| $form_data['destination_terminal_id'] == '-1')
					$warning['destination_terminal_id'] = esc_html__('The terminal is not selected', 'baikalsr-shipping');					
			}			
			if(isset($form_data['destination_to_address']) && $form_data['destination_to_address'] == '1'){
				if(!isset($form_data['destination_street_guid']) || $form_data['destination_street_guid'] == '')
					$warning['destination_street'] = esc_html__('The street is not specified', 'baikalsr-shipping');		
				if(!isset($form_data['destination_street']) || $form_data['destination_street'] == '')
					$warning['destination_street'] = esc_html__('The street is not specified', 'baikalsr-shipping');		
				//Можно вводить дом без классификатора
				//if(!isset($form_data['destination_house_guid']) || $form_data['destination_house_guid'] == '')
				//	$warning['destination_house'] = esc_html__('The house is not specified', 'baikalsr-shipping');		
				if(!isset($form_data['destination_house']) || $form_data['destination_house'] == '')
					$warning['destination_house'] = esc_html__('The house is not specified', 'baikalsr-shipping');	
				if(isset($form_data['destination_prr']) && $form_data['destination_prr'] == '1'){
					if(!isset($form_data['destination_floor']) || $form_data['destination_floor'] == '')
						$warning['destination_floor'] = esc_html__('Floor not specified', 'baikalsr-shipping');	
				}
			}	
			if(!isset($form_data['cargotype_id']) || $form_data['cargotype_id'] == '' 
				|| $form_data['cargotype_id'] == '-1')
				$warning['cargotype'] = esc_html__('The type of cargo is not selected', 'baikalsr-shipping');			
			if(!isset($form_data['volume']) || $form_data['volume'] == '' 
				|| $form_data['volume'] == '0')
				$warning['volume'] = esc_html__('The total volume is not specified', 'baikalsr-shipping');		
			if(!isset($form_data['weight']) || $form_data['weight'] == '' 
				|| $form_data['weight'] == '0')
				$warning['weight'] = esc_html__('The total weight is not specified', 'baikalsr-shipping');					

			return $warning;
		}
		
		/**
		* Hide fields on the order edit page
		*/
		public function admin_shipping_hidden_meta( $hiddenMeta ){
			$hiddenMeta[] = '_destination_guid';
			$hiddenMeta[] = '_destination_name';
			$hiddenMeta[] = '_smethod';
			$hiddenMeta[] = '_terminal_id';
			$hiddenMeta[] = '_terminal_name';
			$hiddenMeta[] = '_terminal_address';
			$hiddenMeta[] = '_request_id';
			$hiddenMeta[] = '_tracking_number';
			$hiddenMeta[] = '_status_name';
			$hiddenMeta[] = '_actual_cost';
			return $hiddenMeta;
		}	
		
		/**
		 * @param WC_Shipping_Rate $rate .
		 * @param int              $index  .
		 */
		public function display_description_if_present($rate) {
			if ( $rate->id != 'baikalsr_shipping_pickup' ) {
				return;
			}
			$shipping_methods = WC()->shipping()->get_shipping_methods();	
			foreach ( $shipping_methods as $id => $shipping_method ) {
				if($id == $rate->get_method_id()){
					if ( isset( $shipping_method->enabled ) && 'yes' === $shipping_method->enabled ) {
						if ( isset($shipping_method->settings['show_terminals']) && $shipping_method->settings['show_terminals'] == 'yes'){
							$this->add_affiliate_changer_btn($rate);
							$this->add_affiliate_data_form($rate);
						}				
					}
				}
			}

		}
		
		public function add_affiliate_changer_btn($method) {
			$template_args = array();
			$meta_data = $method->get_meta_data();
			if ( isset( $meta_data['_preferred_terminal'] ) && ! empty( $meta_data['_preferred_terminal'] ) ) {	
				$template_args['preferred_terminal_id'] = $meta_data['_preferred_terminal']['id'];
				$template_args['preferred_terminal_name'] = $meta_data['_preferred_terminal']['name'];
				$template_args['preferred_terminal_address'] = $meta_data['_preferred_terminal']['address'];	
				wc_get_template( 'checkout/bs-affiliate-button.php', $template_args, '', BS_PLUGIN_DIR_PATH . '/templates/' );
			}
		}

		public function add_affiliate_data_form($method) {
			$template_args = array();
			$template_args['logo_img'] = BS_PLUGIN_DIR_URL . '/assets/img/baikalsr.png';
			$meta_data = $method->get_meta_data();
			if ( isset( $meta_data['_preferred_terminal'] ) && ! empty( $meta_data['_preferred_terminal'] ) ) {
				$template_args['preferred_terminal_id'] = $meta_data['_preferred_terminal']['id'];
			}
			if ( isset( $meta_data['_destination_name'] ) && ! empty( $meta_data['_destination_name'] ) ) {
				$template_args['affiliate_name'] = $meta_data['_destination_name'];
			}
			if ( isset( $meta_data['_terminals'] ) && ! empty( $meta_data['_terminals'] ) ) {
				$template_args['terminals'] = [];
				foreach($meta_data["_terminals"] as $terminal){
					$terminal['schedule'] = Tools::groupSchedule([
						['day' => esc_html__('Mon', 'baikalsr-shipping'), 'hours' => $terminal['schedule'][0]], 
						['day' => esc_html__('Tue', 'baikalsr-shipping'), 'hours' => $terminal['schedule'][1]], 
						['day' => esc_html__('Wed', 'baikalsr-shipping'), 'hours' => $terminal['schedule'][2]], 
						['day' => esc_html__('Thu', 'baikalsr-shipping'), 'hours' => $terminal['schedule'][3]], 
						['day' => esc_html__('Fri', 'baikalsr-shipping'), 'hours' => $terminal['schedule'][4]], 
						['day' => esc_html__('Sat', 'baikalsr-shipping'), 'hours' => $terminal['schedule'][5]], 
						['day' => esc_html__('Sun', 'baikalsr-shipping'), 'hours' => $terminal['schedule'][6]], 
					]);
					$template_args['terminals'][] = $terminal;
				}
			}
			wc_get_template( 'checkout/bs-affiliate-data.php', $template_args, '', BS_PLUGIN_DIR_PATH . '/templates/' );	
		}
		
		public function init_shipping ()
		{	
			if (!class_exists ('\Baikalsr\ShippingMethod'))
			{
				require_once ('ShippingMethod.php');
				$this->shippingMethod = new \Baikalsr\ShippingMethod();
			}
		}
		
		public function backend_enqueue_scripts() {
			$dismiss_data = array( 
				'ajax_url' => admin_url( 'admin-ajax.php' ),
				'security' => wp_create_nonce( 'bs-dismiss-notice' ),
				'no_data' => esc_html__( 'No Data', 'baikalsr-shipping'),
				'send_request' => esc_html__( 'Send a request', 'baikalsr-shipping'),
				'entry_fio' => esc_html__( 'Full Name', 'baikalsr-shipping'),
				'entry_inn_kpp' => esc_html__( 'TIN/IC', 'baikalsr-shipping'),
				'entry_inn' => esc_html__( 'TIN', 'baikalsr-shipping'),
				'err_title' => esc_html__( 'Error', 'baikalsr-shipping'),
				'warning_title' => esc_html__( 'Warning', 'baikalsr-shipping'),
				'entry_contact_person' => esc_html__( 'Contact person', 'baikalsr-shipping'),				
				'min_term' => esc_html__( 'minimum shipping term', 'baikalsr-shipping'),
				'shipping_cost' => esc_html__( 'Shipping cost', 'baikalsr-shipping'),
				'err_shipping_cost' => esc_html__( 'To calculate your shipping costs, you\'ll need to specify your package dimensions, weight, departure, and destination points.', 'baikalsr-shipping'),
			);

			wp_enqueue_script( 'baikalsr-js', BS_PLUGIN_DIR_URL . '/assets/js/baikalsr.js', array('jquery'), BS_VERSION );
			wp_localize_script( 'baikalsr-js', 'baikalsr_js', $dismiss_data );
			
			wp_enqueue_style( 'bs-popup-backend', BS_PLUGIN_DIR_URL . '/assets/css/bs-backend.css', array(), BS_VERSION );
			
			if( true ) {
				// Enqueue Fancybox
				wp_enqueue_script( 'baikalsr-fancybox-js', 'https://cdnjs.cloudflare.com/ajax/libs/fancybox/3.3.5/jquery.fancybox.min.js', array('jquery') );
				wp_enqueue_style( 'baikalsr-fancybox-css', 'https://cdnjs.cloudflare.com/ajax/libs/fancybox/3.3.5/jquery.fancybox.min.css', BS_VERSION );
			}
		}
		
		public function frontend_enqueue_scripts() {
			$dismiss_data = array( 
				'ajax_url' => admin_url( 'admin-ajax.php' ),
			);

			wp_enqueue_script( 'baikalsr-frontend', BS_PLUGIN_DIR_URL . '/assets/js/baikalsr-frontend.js', array('jquery'), BS_VERSION );
			wp_localize_script( 'baikalsr-frontend', 'baikalsr_js', $dismiss_data );
			
			wp_enqueue_style( 'bs-checkout-frontend', BS_PLUGIN_DIR_URL . '/assets/css/bs-frontend.css', array(), BS_VERSION );
			
			if( true ) {
				// Enqueue Fancybox
				wp_enqueue_script( 'baikalsr-fancybox-js', 'https://cdnjs.cloudflare.com/ajax/libs/fancybox/3.3.5/jquery.fancybox.min.js', array('jquery') );
				wp_enqueue_style( 'baikalsr-fancybox-css', 'https://cdnjs.cloudflare.com/ajax/libs/fancybox/3.3.5/jquery.fancybox.min.css', BS_VERSION );
			}
		}		

		/**
		 * Display settings link
		 *
		 * @param array $links Key - link pair.
		 *
		 * @return array
		 */
		public function plugin_action_links( $links ) {
			$settings = array( 'settings' => '<a href="' . admin_url( 'admin.php?page=wc-settings&tab=shipping&section='.self::METHOD_ID ) . '">' . esc_html__( 'Settings', 'baikalsr-shipping' ) . '</a>' );
			$links = $settings + $links;

			return $links;
		}
		
		/**
		 * Этот метод вызывается при запросах (например, Ajax) к плагину из административной части сайта.
		 *
		 * @param string $name Имя плагина
		 * @return $render mixed Null В качестве результата возвращает текст (HTML) или JSON.
		 */	
		public function plgWcOnSelfCallBE() {
			//check_ajax_referer( 'bs-api', 'bs_nonce' );	
			$callback = isset( $_REQUEST['callback'] ) ? wc_clean( $_REQUEST['callback'] ) : '';
			try {			
				switch ($callback) {
					case 'getCities':					
						$json = [];
						try {
							$query = isset( $_GET['term'] ) ? wc_clean( $_GET['term'] ) : '';
							if(!empty($query)){
								$response = Tools::getCitiesByName($query, "");
								if (!empty($response) && is_array($response))
									foreach($response as $item){
										$json[] = array(
											"id"=>$item["guid"],
											"label"=>!empty($item['parents']) ? $item['title'] . ' (' . $item['parents'] . ')' : $item['title'], 
											"value"=>$item['title']
										);
									}	
							}
							wp_send_json( $json );
						} catch (Exception $e) {
							wp_send_json( $json );
							//wp_send_json( array( 'error' => $e->getMessage() ) );
						}
						wp_die();
						break;
					case 'getStreets':	
						$json = [];
						try {
							$query = isset( $_GET['term'] ) ? wc_clean( $_GET['term'] ) : '';
							$guid = isset( $_GET['localityGuid'] ) ? wc_clean( $_GET['localityGuid'] ) : '';
							if(!empty($guid) && !empty($query)){				
								$response = Tools::getStreetsByName($guid, $query);
								if (!empty($response) && is_array($response))
									foreach($response as $item){
										$json[] = array(
											"id"=>$item["guid"],
											"label"=>!empty($item['parents']) ? $item['title'] . ' (' . $item['parents'] . ')' : $item['title'], 
											"value"=>$item['title']
										);
									}	
							}
							wp_send_json( $json );
						} catch (Exception $e) {
							wp_send_json( $json );
							//wp_send_json( array( 'error' => $e->getMessage() ) );
						}
						wp_die();
						break;
					case 'getHouses':	
						$json = [];
						try {
							$query = isset( $_GET['term'] ) ? wc_clean( $_GET['term'] ) : '';
							$guid = isset( $_GET['streetGuid'] ) ? wc_clean( $_GET['streetGuid'] ) : '';
							if(!empty($guid) && !empty($query)){				
								$response = Tools::getHousesByName($guid, $query);
								if (!empty($response) && is_array($response))
									foreach($response as $item){
										$json[] = array(
											"id"=>$item["houseguid"],
											"label"=>$item['formatted'],
											"buildnum"=>$item['buildnum'],
											"strucnum"=>$item['strucnum'],
											"value"=>$item['housenum']
										);
									}
							}
							wp_send_json( $json );
						} catch (Exception $e) {
							wp_send_json( $json );
							//wp_send_json( array( 'error' => $e->getMessage() ) );
						}
						wp_die();
						break;
					case 'getCargoTypes':	
						$json = array('elements' => []);		
						try {
							$apiTypeTest = isset( $_POST['apiTypeTest'] ) ? wc_clean( $_POST['apiTypeTest'] ) : '';
							$apiKey = isset( $_POST['apiKey'] ) ? wc_clean( $_POST['apiKey'] ) : '';
							$response = Tools::getCargoTypes($apiKey, $apiTypeTest);
							if(isset($response['error']) && !empty($response['error'])){
								$json['error'] = $response['error'];
							}else{
								foreach($response as $data) {
									if(isset($data['id']) && isset($data['name']))
										$json['elements'][] = array('id' => $data['id'], 'name' => $data['name']);
								}
							}					
							wp_send_json( $json );
						} catch (Exception $e) {
							wp_send_json( $json );
							//wp_send_json( array( 'error' => $e->getMessage() ) );
						}
						wp_die();
						break;
					case 'getContragents':	
						$json = array('elements' => []);
						try {
							$apiTypeTest = isset( $_POST['apiTypeTest'] ) ? wc_clean( $_POST['apiTypeTest'] ) : '';
							$apiKey = isset( $_POST['apiKey'] ) ? wc_clean( $_POST['apiKey'] ) : '';
							$response = Tools::getContragents($apiKey, $apiTypeTest);
							if(isset($response['error']) && !empty($response['error'])){
								$json['error'] = $response['error'];
							}else{
								foreach($response as $data) {
									if(isset($data['guid']) && isset($data['name']))
										$json['elements'][] = array('id' => $data['guid'], 'name' => $data['name']);
								}
							}					
							wp_send_json( $json );
						} catch (Exception $e) {
							wp_send_json( $json );
							//wp_send_json( array( 'error' => $e->getMessage() ) );
						}
						wp_die();
						break;
					case 'getTerminals':	
						$json = array('elements' => []);
						try {
							$guid = isset( $_POST['localityGuid'] ) ? wc_clean( $_POST['localityGuid'] ) : '';
							$apiTypeTest = isset( $_POST['apiTypeTest'] ) ? wc_clean( $_POST['apiTypeTest'] ) : '';
							$apiKey = isset( $_POST['apiKey'] ) ? wc_clean( $_POST['apiKey'] ) : '';
							$terminal_type = isset( $_POST['terminalType'] ) ? wc_clean( $_POST['terminalType'] ) : 'start';
							$response = Tools::getAffiliateData($guid, $apiKey, $apiTypeTest);
							if(isset($response['error']) && !empty($response['error'])){
								$json['error'] = $response['error'];
							}else{
								if (isset($response["terminals"]) && !empty($response["terminals"])) {
									foreach($response["terminals"] as $terminal) {
										if(isset($terminal[$terminal_type]) && $terminal[$terminal_type]){
											$terminal["address"] = Tools::htmlspecialcharsBack($terminal["address"]);
											$json['elements'][] = array(
												'id' => !empty($terminal['id']) ? $terminal['id'] : 0, 
												'name' => $terminal['name']." (".$terminal["address"].")"
											);
										}
									}
								}
							}				
							wp_send_json( $json );
						} catch (Exception $e) {
							wp_send_json( $json );
							//wp_send_json( array( 'error' => $e->getMessage() ) );
						}
						wp_die();
						break;
					case 'submitCalcForm':
						$json = array();	
						//get shipment config
						$config = WC()->shipping->load_shipping_methods()[self::METHOD_ID];
						//Установим ключ api для выполнения расчета
						if (!empty($config)){
							if($config->get_option( 'test_mode' ) == 'yes'){
								Tools::defineAuthParams( $config->get_option( 'authKeyTest' ), true );
							}else{
								Tools::defineAuthParams( $config->get_option( 'authKey' ), false );
							}				
						}else{
							$json['error'] = esc_html__( 'No Methods Conf', 'baikalsr-shipping');
						}
						//$json['error'] = esc_html__( 'No Methods Conf', 'baikalsr-shipping');
						if(!isset($json['error'])){
							if (isset($_POST['params']) && !empty($_POST['params'])){									
								$json = Tools::calculate($_POST['params']);	
							}else{
								$json['error'] = esc_html__( 'Error Request', 'baikalsr-shipping');
							}
						}
						wp_send_json($json);
						wp_die();
						break;
					case 'submitOrderForm':
						$json = array();	
						$post_data = array();
						if (isset($_POST['params']) && !empty($_POST['params'])){
							$post_data = $_POST['params'];
							if (isset($post_data['order_id']) && !empty($post_data['order_id'])){
								$order_id = $post_data['order_id'];
								$order = wc_get_order($order_id);
								if(empty($order)){
									$json['error'] = esc_html__( 'Order not found', 'baikalsr-shipping');
								}	
								$shipping = NULL;						
								foreach ( $order->get_shipping_methods() as $order_item_shipping ) {
									if ( $order_item_shipping instanceof WC_Order_Item_Shipping && $order_item_shipping->get_method_id() == self::METHOD_ID ) 
										$shipping = $order_item_shipping;
								}	
								if(empty($shipping)){
									$json['error'] = esc_html__('Could not send the request!', 'baikalsr-shipping');
								}	
								if(!isset($json['error'])){
									$popup_data = $this->getShippingPopupData($order, $post_data);	
									ob_start();
									wc_get_template( 'order/bs-popup-shipping.php', $popup_data, '', BS_PLUGIN_DIR_PATH . '/templates/' );
									$json['popup_html'] = ob_get_clean();									
									if(isset($popup_data['success']) && $popup_data['success']){																	
										ob_start();
										wc_get_template( 'order/bs-popup-button.php', $this->getShippingInfoData($shipping), '', BS_PLUGIN_DIR_PATH . '/templates/' );	
										$json['info_html'] = ob_get_clean();	
										$json['success'] = true;	
									}
								}
							}else{
								$json['error'] = esc_html__( 'Order not found', 'baikalsr-shipping');
							}
						}else{	
							$json['error'] = esc_html__( 'Error Request', 'baikalsr-shipping');
						}
						wp_send_json($json);
						wp_die();
						break;
					case 'tracking':
						$json = array();
						$order_id = $_POST['order_id'];
						$order = wc_get_order($order_id);
						if(empty($order)){
							$json['error'] = esc_html__( 'Order not found', 'baikalsr-shipping');
						}	
						$shipping = NULL;						
						foreach ( $order->get_shipping_methods() as $order_item_shipping ) {
							if ( $order_item_shipping instanceof WC_Order_Item_Shipping && $order_item_shipping->get_method_id() == self::METHOD_ID ) 
								$shipping = $order_item_shipping;
						}	
						if(empty($shipping)){
							$json['error'] = esc_html__('Could not send the request!', 'baikalsr-shipping');
						}	
						$request_id	= '';					
						foreach ($shipping->get_meta_data() as $value) {		
							$meta = $value->get_data();
							if($meta['key'] == '_request_id'){
								$request_id = esc_attr( $meta['value']);
							}
						}
						if(empty($request_id)){
							$json['error'] = esc_html__('Could not send the request!', 'baikalsr-shipping');
						}	
						if(!isset($json['error'])){
							//get shipment config
							$config = WC()->shipping->load_shipping_methods()[self::METHOD_ID];
							//Установим ключ api для выполнения расчета
							if (!empty($config)){
								if($config->get_option( 'test_mode' ) == 'yes'){
									Tools::defineAuthParams( $config->get_option( 'authKeyTest' ), true );
								}else{
									Tools::defineAuthParams( $config->get_option( 'authKey' ), false );
								}				
							}else{
								$json['error'] = esc_html__( 'No Methods Conf', 'baikalsr-shipping');
							}
							$baikalResponse = Tools::getOrderData($request_id);
							if(isset($baikalResponse["status"]["name"]) && !empty($baikalResponse["status"]["name"])){
								//$json['status_name'] = $baikalResponse["status"]["name"];
								$shipping->update_meta_data('_status_name', sanitize_text_field( $baikalResponse["status"]["name"]));
							}else{
								$json['error'] = esc_html__('No Response', 'baikalsr-shipping');
							}
							if (isset($baikalResponse["total"]["sum"]) && !empty($baikalResponse["total"]["sum"])) {
								//$json['actual_cost'] = (float) $baikalResponse["total"]["sum"];
								$shipping->update_meta_data('_actual_cost', sanitize_text_field( $baikalResponse["total"]["sum"]));
							}else{
								$json['error'] = esc_html__('No Response', 'baikalsr-shipping');
							}
							$shipping->save_meta_data();	
							ob_start();
							wc_get_template( 'order/bs-popup-button.php', $this->getShippingInfoData($shipping), '', BS_PLUGIN_DIR_PATH . '/templates/' );	
							$json['info_html'] = ob_get_clean();							
						}
						
						wp_send_json($json);
						wp_die();
						break;	
					default:
						//echo esc_html__( 'Error Request', 'baikalsr-shipping');
						return;
				}
			} catch (Exception $e) {
				//wp_send_json( array( 'error' => $e->getMessage() ) );
			}
			wp_die();
		}
	}
}
