<?php
namespace {

    defined('ABSPATH') or exit;
}

namespace Baikalsr {
	
	use Baikalsr\Fields;
	use Baikalsr\Tools;
	use WC_Shipping_Method;
	use Automattic\WooCommerce\Utilities\I18nUtil;	
	
	class ShippingMethod extends WC_Shipping_Method 
	{
		public $baikalsrTools;
		public string $departure_guid;
		
		public function __construct ($instance_id = 0)
		{
			parent::__construct($instance_id);
			 
			$this->id = 'baikalsr_shipping';
			//$this->instance_id = absint ($instance_id);
			$this->method_title = esc_html__('Baikal Service', 'baikalsr-shipping');
			$this->method_description = esc_html__('Shows live Baikal Service domestic rates on checkout', 'baikalsr-shipping');
			$this->enabled = "yes";
			$this->title = esc_html__('BaikalSR Shipping', 'baikalsr-shipping');			
			$this->init ();
		} 
		
		final public function init(): void
		{	
			// Load the settings.
			$this->init_form_fields();
			$this->init_settings();
			
			// Define user set variables.
			$this->baikalsrTools = new Tools();
			if($this->get_option( 'test_mode' ) == 'yes'){
				$this->baikalsrTools::defineAuthParams( $this->get_option( 'authKeyTest' ), true );
			}else{
				$this->baikalsrTools::defineAuthParams( $this->get_option( 'authKey' ), false );
			}
			
			$this->departure_guid = $this->get_option( 'departure_guid', '' );
			
			add_action('woocommerce_update_options_shipping_' . $this->id, [$this, 'process_admin_options']);
			add_filter('woocommerce_settings_api_form_fields_' .$this->id, array( $this, 'after_init_set_field_options' ) );

		}
		
		final public function init_form_fields(): void
		{
			$this->form_fields = [
				'api_group' => [
					'title' => esc_html__('API Settings', 'baikalsr-shipping'),
					'type'  => 'title',
				],
				'authKey' => [
					'title'       => esc_html__('API Key', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => false,
					'desc_tip'    => true,
					'description' => esc_html__('You can get the API key when you register in the personal account', 'baikalsr-shipping'),
				],
				'authKeyTest' => [
					'title'       => esc_html__('API Test Key', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => false,
					'desc_tip'    => true,
					'description' => esc_html__('You can get the API key when you register in the personal account', 'baikalsr-shipping'),
				],
				'test_mode' => [
					'title'       => esc_html__('Test Mode', 'baikalsr-shipping'),
					'type'        => 'checkbox',
					'label'  	  => esc_html__('Enable the Test Mode', 'baikalsr-shipping'),
					'default'     => false,
					'desc_tip'    => true,
					'description' =>  esc_html__('When enabled, the test API is used', 'baikalsr-shipping'),
				],
				'courier_group' => [
					'title' => esc_html__('Courier Shipping', 'baikalsr-shipping'),
					'type'  => 'title',
				],
				'courier_title' => [
					'title'       => esc_html__('Title', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => esc_html__('BS Courier Shipping', 'baikalsr-shipping'),
					'desc_tip'    => true,
					'description' => esc_html__('This controls the title which the customer sees during checkout', 'baikalsr-shipping'),
				],
				'courier_shipping' => [
					'type'        => 'checkbox',
					'label'  	  => esc_html__('Courier Shipping', 'baikalsr-shipping'),
					'default'     => 'yes',
				],
				'pickup_group' => [
					'title' => esc_html__('Pickup Shipping', 'baikalsr-shipping'),
					'type'  => 'title',
				],
				'pickup_title' => [
					'title'       => esc_html__('Title', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => esc_html__('BS Pickup Shipping', 'baikalsr-shipping'),
					'desc_tip'    => true,
					'description' => esc_html__('This controls the title which the customer sees during checkout', 'baikalsr-shipping'),
				],
				'pickup_shipping' => [
					'type'        => 'checkbox',
					'label'  	  => esc_html__('Pickup Shipping', 'baikalsr-shipping'),
					'default'     => 'yes',
				],
				'consignor_group' => [
					'title' => esc_html__('Consignor', 'baikalsr-shipping'),
					'type'  => 'title',
				],
				'consignor_contragent_id' => [
					'title'       => esc_html__('Counterparty', 'baikalsr-shipping'),
					'type'        => 'select',
					'default'     => false,
					'class'    	  => 'wc-enhanced-select',
					'css'         => 'min-width:300px;',
					'desc_tip'    => true,
					'description' => 'Please select a counterparty from the list and specify contacts for feedback.',
					'custom_attributes' => [
						'data-url'=> 'getContragents',
					],
				],
				'author_lastname' => [
					'title'       => esc_html__('Last Name', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => false,
				],
				'author_firstname' => [
					'title'       => esc_html__('First Name', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => false,
				],
				'author_middlename' => [
					'title'       => esc_html__('Middle Name', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => false,
				],
				'consignor_phone' => [
					'title'       => esc_html__('Phone', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => false,
				],
				'consignor_email' => [
					'title'       => esc_html__('Email', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => false,
					'custom_attributes' => [
                        'type'     => 'email',
                    ],
				],
				'departure_group' => [
					'title' => esc_html__('Ship From', 'baikalsr-shipping'),
					'type'  => 'title',
				],
				'departure' => [
					'id'       	  => 'woocommerce_baikalsr_shipping_departure',
					'title'       => esc_html__('Departure', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => false,	
					'custom_attributes' => [
						'data-url'=> 'getCities',
						'data-title'=> '',
						'data-chars'=> 2,
						'primary'=> true,
						'related_field'=> 'departure_guid',
					],
					'desc_tip'    => false,
					'description' => '',
				],
				'departure_guid' => [
					'title'       => esc_html__('Departure GUID', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => false,
					'custom_attributes' => [
						'secondary'=> true,
					],
				],
				'departure_from_address' => [
					'id'       	  => 'woocommerce_baikalsr_shipping_departure_from_address',
					'title'       => esc_html__('Departure type', 'baikalsr-shipping'),
					'type'        => 'select',
					'class'    	  => 'wc-enhanced-select',
					'css'         => 'min-width:300px;',
					'default'     => false,
					'options'  => array(
						1 => esc_html__('Pickup', 'baikalsr-shipping'),
						0 => esc_html__('Drop-off', 'baikalsr-shipping'),		
					),
				],
				'departure_terminal_id' => [
					'id'       	  => 'woocommerce_baikalsr_shipping_departure_terminal_id',
					'title'       => esc_html__('Address of terminal', 'baikalsr-shipping'),
					'type'        => 'select',
					'class'    	  => 'wc-enhanced-select',
					'css'         => 'min-width:300px;',
					'default'     => false,
					'custom_attributes' => [
						'hide_if_checked'=> 'departure_from_address',
						'data-url'=> 'getTerminals',				
					],
				],
				'departure_street' => [
					'id'       	  => 'woocommerce_baikalsr_shipping_departure_street',
					'title'       => esc_html__('Street', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => false,	
					'custom_attributes' => [
						'show_if_checked'=> 'departure_from_address',
						'data-url'=> 'getStreets',
						'data-title'=> '',
						'data-chars'=> 2,
						'primary'=> true,
						'related_field'=> 'departure_street_guid',
					],
					'desc_tip'    => false,
					'description' => '',
				],
				'departure_street_guid' => [
					'title'       => esc_html__('Street GUID', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => false,
					'custom_attributes' => [
						'show_if_checked'=> 'departure_from_address',
						'secondary'=> true,
					],
				],
				'departure_house' => [
					'id'       	  => 'woocommerce_baikalsr_shipping_departure_house',
					'title'       => esc_html__('House', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => false,	
					'custom_attributes' => [
						'show_if_checked'=> 'departure_from_address',
						'data-url'=> 'getHouses',
						'data-title'=> '',
						'data-chars'=> 1,
						'primary'=> true,
						'related_field'=> 'departure_house_guid',
					],
				],
				'departure_house_guid' => [
					'title'       => esc_html__('House GUID', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => false,
					'custom_attributes' => [
						'show_if_checked'=> 'departure_from_address',
						'secondary'=> true,
					],
				],
				'departure_housing' => [
					'id'       	  => 'woocommerce_baikalsr_shipping_departure_housing',
					'title'       => esc_html__('Housing', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => false,
					'custom_attributes' => [
						'show_if_checked'=> 'departure_from_address',
					]
				],
				'departure_building' => [
					'id'       	  => 'woocommerce_baikalsr_shipping_departure_building',
					'title'       => esc_html__('Building', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => false,
					'custom_attributes' => [
						'show_if_checked'=> 'departure_from_address',
					]
				],
				'departure_apartment' => [
					'id'       	  => 'woocommerce_baikalsr_shipping_departure_apartment',
					'title'       => esc_html__('Apartment', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => false,
					'custom_attributes' => [
						'show_if_checked'=> 'departure_from_address',
					]
				],
				'departure_comment' => [
					'id'       	  => 'woocommerce_baikalsr_shipping_departure_comment',
					'title'       => esc_html__('Comment', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => false,
					'custom_attributes' => [
						'show_if_checked'=> 'departure_from_address',
					]
				],
				'departure_prr_group' => [
					'title' => esc_html__('Departure Loader Services', 'baikalsr-shipping'),
					'type'  => 'title',
					'custom_attributes' => [
						'show_if_checked'=> 'departure_from_address',
					]
				],
				'departure_prr' => [
					'id'       	  => 'woocommerce_baikalsr_shipping_departure_prr',
					'title'       => esc_html__('Loader Services', 'baikalsr-shipping'),
					'type'        => 'checkbox',
					'label'  	  => esc_html__('Loader Services is required', 'baikalsr-shipping'),
					'default'     => false,
					'custom_attributes' => [
						'show_if_checked'=> 'departure_from_address',
					]
				],
				'departure_floor' => [
					'id'       	  => 'woocommerce_baikalsr_shipping_departure_floor',
					'title'       => esc_html__('Floor', 'baikalsr-shipping'),
					'type'        => 'number',			
					'default'     => 1,
					'custom_attributes' => [
						'show_if_checked'=> 'departure_from_address',
						'show_if_checked'=> 'departure_prr',	
						'min'         => 1,
						'step'        => 1,						
					]
				],
				'departure_lift' => [
					'id'       	  => 'woocommerce_baikalsr_shipping_departure_lift',
					'title'       => esc_html__('Elevator', 'baikalsr-shipping'),
					'type'        => 'checkbox',
					'label'  	  => esc_html__('There is an elevator', 'baikalsr-shipping'),
					'default'     => false,
					'custom_attributes' => [
						'show_if_checked'=> 'departure_from_address',
						'show_if_checked'=> 'departure_prr',			
					]
				],
				'default_group' => [
					'title' => esc_html__('Default Settings', 'baikalsr-shipping'),
					'type'  => 'title',
				],
				'cargotype_id' => [
					'id'       	  => 'woocommerce_baikalsr_shipping_cargotype_id',
					'title'       => esc_html__('Cargo Type', 'baikalsr-shipping'),
					'type'        => 'select',
					'class'    	  => 'wc-enhanced-select',
					'css'         => 'min-width:300px;',
					'default'     => false,
					'desc_tip'    => true,
					'description' => esc_html__('Dangerous goods, liquids, food, animals and plants are not allowed for transportation', 'baikalsr-shipping'),
					'custom_attributes' => [
						'data-url'=> 'getCargoTypes',
					],
				],
				'packing_id' => [
					'id'       	  => 'woocommerce_baikalsr_shipping_packing_id',
					'title'       => esc_html__('Packing', 'baikalsr-shipping'),
					'type'        => 'select',
					'class'    	  => 'wc-enhanced-select',
					'css'         => 'min-width:300px;',
					'default'     => 0,
					'desc_tip'    => true,
					'description' => esc_html__('Any cargo must be packed in accordance with the rules of transportation — this will ensure its safety during the journey. Type of packaging depending on the nature of the cargo.', 'baikalsr-shipping'),
					'custom_attributes' => [
						'data-url'=> 'getPacking',
					],
				],	
				'lengthD' => [
					'id'       	  => 'woocommerce_baikalsr_shipping_lengthD',
					'title'       => esc_html__('Length Default', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => '0.3',
				],
				'widthD' => [
					'id'       	  => 'woocommerce_baikalsr_shipping_widthD',
					'title'       => esc_html__('Width Default', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => '0.4',
				],
				'heightD' => [
					'title'       => esc_html__('Height Default', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => '0.2',
				],
				'lwh_unit' => [
					'title'    => esc_html__( 'Dimensions unit', 'woocommerce' ),
					'desc'     => esc_html__( 'This controls what unit you will define lengths in.', 'woocommerce' ),
					'id'       => 'woocommerce_dimension_unit',
					'class'    => 'wc-enhanced-select',
					'css'      => 'min-width:300px;',
					'default'  => 'm',
					'type'     => 'select',
					'options'  => array(
						'm'  => I18nUtil::get_dimensions_unit_label( 'm' ),
						'cm' => I18nUtil::get_dimensions_unit_label( 'cm' ),
						'mm' => I18nUtil::get_dimensions_unit_label( 'mm' ),
						'in' => I18nUtil::get_dimensions_unit_label( 'in' ),
						'yd' => I18nUtil::get_dimensions_unit_label( 'yd' ),
					),
					'desc_tip' => true,
				],				
				'weightD' => [
					'title'       => esc_html__('Weight Default', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => '1',
				],
				'weight_unit' => [
					'title'    => esc_html__( 'Weight unit', 'woocommerce' ),
					'desc'     => esc_html__( 'This controls what unit you will define weights in.', 'woocommerce' ),
					'id'       => 'woocommerce_weight_unit',
					'class'    => 'wc-enhanced-select',
					'css'      => 'min-width:300px;',
					'default'  => 'kg',
					'type'     => 'select',
					'options'  => array(
						'kg'  => I18nUtil::get_weight_unit_label( 'kg' ),
						'g'   => I18nUtil::get_weight_unit_label( 'g' ),
						'lbs' => I18nUtil::get_weight_unit_label( 'lbs' ),
						'oz'  => I18nUtil::get_weight_unit_label( 'oz' ),
					),
					'desc_tip' => true,
				],				
				'default_for_item' => [
					'title'       => esc_html__('Applying the Default Values', 'baikalsr-shipping'),
					'type'        => 'select',
					'class'       => 'wc-enhanced-select',
					'css'         => 'min-width:300px;',
					'default'     => '0',
					'desc_tip'    => true,
					'description' => esc_html__('Set how the default dimensions and weights will be applied', 'baikalsr-shipping'),
					'options' => array(
						0 => esc_html__('For Total Weight and Volume', 'baikalsr-shipping'),
						1 => esc_html__('For Product Unit', 'baikalsr-shipping'),
					),
				],
				'additional_group' => [
					'title' => esc_html__('Additional Settings', 'baikalsr-shipping'),
					'type'  => 'title',
				],
				'additional_cost' => [
					'title'       => esc_html__('Additional Cost', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => '',
				],
				'additional_cost_percent' => [
					'title'       => esc_html__('Additional Cost, %', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => '',
				],
				'free_shipment' => [
					'title'       => esc_html__('Min. Total for Free Shipping', 'baikalsr-shipping'),
					'type'        => 'text',
					'default'     => '',
				],
				/*'blocked_destinations' => array(
					'title'          => esc_html__( 'Blocked destinations', 'baikalsr-shipping' ),
					'type'           => 'multiselect',
					'default' 		 => 'cod',
					'desc_tip'       => false,
					'options'        => [],
					'class'          => 'wc-enhanced-select',
				),*/
				'show_term' => [
					'title'       => esc_html__('Display terms', 'baikalsr-shipping'),
					'type'        => 'checkbox',
					'label'  	  => esc_html__('Display delivery terms on checkout', 'baikalsr-shipping'),
					'default'     => 'yes',
				],
				'show_terminals' => [
					'title'       => esc_html__('Display terminal', 'baikalsr-shipping'),
					'type'        => 'checkbox',
					'label'  	  => esc_html__('Display terminal selection on checkout', 'baikalsr-shipping'),
					'default'     => 'yes'
				],
			];
		}
		
		/**
		 * Generate Checkbox HTML.
		 *
		 * @param string $key Field key.
		 * @param array  $data Field data.
		 * @return string
		 */
		public function generate_checkbox_html( $key, $data ) {
			$field_key = $this->get_field_key( $key );
			$defaults  = array(
				'title'             => '',
				'label'             => '',
				'disabled'          => false,
				'class'             => '',
				'css'               => '',
				'type'              => 'text',
				'desc_tip'          => false,
				'description'       => '',
				'custom_attributes' => array(),
			);

			$data = wp_parse_args( $data, $defaults );

			if ( ! $data['label'] ) {
				$data['label'] = $data['title'];
			}
			
			$hidden_option = $this->hiding_check( $data['custom_attributes'] );
			unset($data['custom_attributes']['show_if_checked']);
			unset($data['custom_attributes']['hide_if_checked']);
			ob_start();
			?>
			<tr valign="top" style="<?= $hidden_option ? 'display:none;' : ''?>">
				<th scope="row" class="titledesc">
					<label for="<?php echo esc_attr( $field_key ); ?>"><?php echo wp_kses_post( $data['title'] ); ?> <?php echo $this->get_tooltip_html( $data ); // WPCS: XSS ok. ?></label>
				</th>
				<td class="forminp">
					<fieldset>
						<legend class="screen-reader-text"><span><?php echo wp_kses_post( $data['title'] ); ?></span></legend>
						<label for="<?php echo esc_attr( $field_key ); ?>">
						<input <?php disabled( $data['disabled'], true ); ?> class="<?php echo esc_attr( $data['class'] ); ?>" type="checkbox" name="<?php echo esc_attr( $field_key ); ?>" id="<?php echo esc_attr( $field_key ); ?>" style="<?php echo esc_attr( $data['css'] ); ?>" value="1" <?php checked( $this->get_option( $key ), 'yes' ); ?> <?php echo $this->get_custom_attribute_html( $data ); // WPCS: XSS ok. ?> /> <?php echo wp_kses_post( $data['label'] ); ?></label><br/>
						<?php echo $this->get_description_html( $data ); // WPCS: XSS ok. ?>
					</fieldset>
				</td>
			</tr>
			<?php

			return ob_get_clean();
		}
		
		/**
		 * Generate Text Input HTML.
		 *
		 * @param string $key Field key.
		 * @param array  $data Field data.
		 * @return string
		 */
		public function generate_text_html( $key, $data ) {
			$field_key = $this->get_field_key( $key );
			$defaults  = array(
				'title'             => '',
				'disabled'          => false,
				'class'             => '',
				'css'               => '',
				'placeholder'       => '',
				'type'              => 'text',
				'desc_tip'          => false,
				'description'       => '',
				'custom_attributes' => array(),
			);

			$data = wp_parse_args( $data, $defaults );
			if(isset($data['custom_attributes']['data-title'])){
				$data['custom_attributes']['data-title'] = esc_attr( $this->get_option( $key ) );
			}	
			if(isset($data['custom_attributes']['primary']) && $data['custom_attributes']['primary']){
				if(isset($data['custom_attributes']['related_field']) && !empty($data['custom_attributes']['related_field'])){
					$related_key = $data['custom_attributes']['related_field'];
					$related_field_key = $this->get_field_key($data['custom_attributes']['related_field']);
				}
			}
			if(isset($data['custom_attributes']['secondary']) && $data['custom_attributes']['secondary']){
				return;
			}			
			$hidden_option = $this->hiding_check( $data['custom_attributes'] );
			unset($data['custom_attributes']['primary']);
			unset($data['custom_attributes']['secondary']);
			unset($data['custom_attributes']['related_field']);
			unset($data['custom_attributes']['show_if_checked']);
			unset($data['custom_attributes']['hide_if_checked']);
			ob_start();
			?>
			<tr valign="top" style="<?= $hidden_option ? 'display:none;' : ''?>">
				<th scope="row" class="titledesc">
					<label for="<?php echo esc_attr( $field_key ); ?>"><?php echo wp_kses_post( $data['title'] ); ?> <?php echo $this->get_tooltip_html( $data ); // WPCS: XSS ok. ?></label>
				</th>
				<td class="forminp">
					<fieldset>
						<legend class="screen-reader-text"><span><?php echo wp_kses_post( $data['title'] ); ?></span></legend>
						<?php if(isset($related_field_key) && !empty($related_field_key)): ?>
						<input type="hidden" name="<?php echo esc_attr( $related_field_key ); ?>" id="<?php echo esc_attr( $related_field_key ); ?>" value="<?php echo esc_attr( $this->get_option( $related_key ) ); ?>"/>
						<?php endif; ?>
						<input class="input-text regular-input <?php echo esc_attr( $data['class'] ); ?>" type="<?php echo esc_attr( $data['type'] ); ?>" name="<?php echo esc_attr( $field_key ); ?>" id="<?php echo esc_attr( $field_key ); ?>" style="<?php echo esc_attr( $data['css'] ); ?>" value="<?php echo esc_attr( $this->get_option( $key ) ); ?>" placeholder="<?php echo esc_attr( $data['placeholder'] ); ?>" <?php disabled( $data['disabled'], true ); ?> <?php echo $this->get_custom_attribute_html( $data ); // WPCS: XSS ok. ?> />
						<?php echo $this->get_description_html( $data ); // WPCS: XSS ok. ?>
					</fieldset>
				</td>
			</tr>
			<?php

			return ob_get_clean();
		}
		
		/**
		 * Generate Select HTML.
		 *
		 * @param string $key Field key.
		 * @param array  $data Field data.
		 * @return string
		 */
		public function generate_select_html( $key, $data ) {
			$field_key = $this->get_field_key( $key );
			$defaults  = array(
				'title'             => '',
				'disabled'          => false,
				'class'             => '',
				'css'               => '',
				'placeholder'       => '',
				'type'              => 'text',
				'desc_tip'          => false,
				'description'       => '',
				'custom_attributes' => array(),
				'options'           => array(),
			);

			$data  = wp_parse_args( $data, $defaults );
			$value = $this->get_option( $key );
			
			$hidden_option = $this->hiding_check( $data['custom_attributes'] );
			unset($data['custom_attributes']['show_if_checked']);
			unset($data['custom_attributes']['hide_if_checked']);
			
			ob_start();
			?>
			<tr valign="top" style="<?= $hidden_option ? 'display:none;' : ''?>">
				<th scope="row" class="titledesc">
					<label for="<?php echo esc_attr( $field_key ); ?>"><?php echo wp_kses_post( $data['title'] ); ?> <?php echo $this->get_tooltip_html( $data ); // WPCS: XSS ok. ?></label>
				</th>
				<td class="forminp">
					<fieldset>
						<legend class="screen-reader-text"><span><?php echo wp_kses_post( $data['title'] ); ?></span></legend>
						<select class="select <?php echo esc_attr( $data['class'] ); ?>" name="<?php echo esc_attr( $field_key ); ?>" id="<?php echo esc_attr( $field_key ); ?>" style="<?php echo esc_attr( $data['css'] ); ?>" <?php disabled( $data['disabled'], true ); ?> <?php echo $this->get_custom_attribute_html( $data ); // WPCS: XSS ok. ?>>
							<?php foreach ( (array) $data['options'] as $option_key => $option_value ) : ?>
								<?php if ( is_array( $option_value ) ) : ?>
									<optgroup label="<?php echo esc_attr( $option_key ); ?>">
										<?php foreach ( $option_value as $option_key_inner => $option_value_inner ) : ?>
											<option value="<?php echo esc_attr( $option_key_inner ); ?>" <?php selected( (string) $option_key_inner, esc_attr( $value ) ); ?>><?php echo esc_html( $option_value_inner ); ?></option>
										<?php endforeach; ?>
									</optgroup>
								<?php else : ?>
									<option value="<?php echo esc_attr( $option_key ); ?>" <?php selected( (string) $option_key, esc_attr( $value ) ); ?>><?php echo esc_html( $option_value ); ?></option>
								<?php endif; ?>
							<?php endforeach; ?>
						</select>
						<?php echo $this->get_description_html( $data ); // WPCS: XSS ok. ?>
					</fieldset>
				</td>
			</tr>
			<?php

			return ob_get_clean();
		}
		
		/**
		 * Generate Title HTML.
		 *
		 * @param string $key Field key.
		 * @param array  $data Field data.
		 * @since  1.0.0
		 * @return string
		 */
		public function generate_title_html( $key, $data ) {
			$field_key = $this->get_field_key( $key );
			$defaults  = array(
				'title' => '',
				'class' => '',
				'custom_attributes' => array(),
			);

			$data = wp_parse_args( $data, $defaults );
			
			$hidden_option = $this->hiding_check( $data['custom_attributes'] );
			unset($data['custom_attributes']['show_if_checked']);
			unset($data['custom_attributes']['hide_if_checked']);
			
			ob_start();
			?>
				</table>
				<h3 class="wc-settings-sub-title <?php echo esc_attr( $data['class'] ); ?>" id="<?php echo esc_attr( $field_key ); ?>" style="<?= $hidden_option ? 'display:none;' : ''?>"><?php echo wp_kses_post( $data['title'] ); ?></h3>
				<?php if ( ! empty( $data['description'] ) ) : ?>
					<p><?php echo wp_kses_post( $data['description'] ); ?></p>
				<?php endif; ?>
				<table class="form-table">
			<?php

			return ob_get_clean();
		}
	
		//check if the field needs to be hidden 
		public function hiding_check( $custom_attributes ) {
			$hidden_option = false;
			if ( ! empty( $custom_attributes ) && is_array( $custom_attributes ) ) {
				foreach ( $custom_attributes as $attribute => $attribute_value ) {
					if ( $attribute == 'show_if_checked' || $attribute == 'hide_if_checked' ) {
						$option_value = esc_attr($this->get_option( $attribute_value ));
						if ( $option_value == 'yes' || $option_value == 'no') {
							$option_value = $option_value == 'yes' ? true : false;
						}
						if ( $attribute == 'show_if_checked'  && !$option_value) {
							$hidden_option = true;
						}
						if ( $attribute == 'hide_if_checked' && $option_value) {
							$hidden_option = true;
						}
					}
				}
			}
			return $hidden_option;
		}
		
		// Set specific field options after initialization
		public function after_init_set_field_options ( $fields ) {
			$bsFields = new Fields();
			if ( isset( $fields['departure_terminal_id'] )) {
				$fields['departure_terminal_id']['options'] = $bsFields->getTerminals($this->departure_guid);
			}
			if ( isset( $fields['packing_id'] ) ) {
				$fields['packing_id']['options'] = $bsFields->getPacking();
			}
			if ( isset( $fields['consignor_contragent_id'] ) ) {
				$fields['consignor_contragent_id']['options'] = $bsFields->getContragents();
			}
			if ( isset( $fields['cargotype_id'] ) ) {
				$fields['cargotype_id']['options'] = $bsFields->getCargoTypes();
			}
			if ( isset( $fields['blocked_destinations'] ) ) {
				$blocked_destinations = [];
				if ( isset( $fields['blocked_destinations'] ) ) {
					$fields['blocked_destinations']['options'] = $blocked_destinations;
				}
			}
			return $fields;
		}
		
		/**
		 * calculate_shipping function.
		 *
		 * @access public
		 * @param array $package
		 * @return void
		 */
		public function calculate_shipping( $package = array() ) {
			$this->baikalsrTools = new Tools();
			$total_price = $this->baikalsrTools::intOrFloat($package['contents_cost']);
			$total_dimensions = $this->_getDimensionsAndWeight($package['contents'], $this->settings);
			$destination = $this->baikalsrTools::_getLocationWithGUIDFromAddress($package['destination']);
			if(isset($destination["guid"]) && !empty($destination["guid"])){
				$preorderData = array(
					'departure_guid' => $this->settings['departure_guid'],
					'departure_from_address' => $this->settings['departure_from_address'],
					'departure_street_guid' => $this->settings['departure_street_guid'],
					'departure_house' => $this->settings['departure_house'],
					'departure_housing' => $this->settings['departure_housing'],
					'departure_building' => $this->settings['departure_building'],
					'departure_prr' => $this->settings['departure_prr']== 'yes' ? true : false,
					'departure_floor' => $this->settings['departure_floor'],
					'departure_lift' => $this->settings['departure_lift']== 'yes' ? true : false,
					'destination_guid' => $destination["guid"],
					//'destination_affiliate_guid' => $destination['affiliate']['guid'] ? $destination['affiliate']['guid'] : $destination['guid'],
					'destination_to_address' => true,
					'estimated_cost' => $this->baikalsrTools::intOrFloat($package['contents_cost']),
					'packing_id' => $this->settings['packing_id'],
					'weight' => $total_dimensions["weight"],
					'length' => $total_dimensions["length"],
					'width' => $total_dimensions["width"],
					'height' => $total_dimensions["height"],
					'volume' => $total_dimensions["volume"],
				);
				$calcResult = $this->baikalsrTools::_calculate($preorderData);
				if (isset($calcResult["total"]) && !empty($calcResult["total"])){
					if(empty($this->settings['additional_cost'])) $this->settings['additional_cost'] = 0.0;
					if(empty($this->settings['additional_cost_percent'])) $this->settings['additional_cost_percent'] = 0.0;
								
					$destination_total = 0;
					if(isset($calcResult["destination"]["total"]) && !empty($calcResult["total"]))
						$destination_total = (float)$calcResult["destination"]["total"];
					$id = $this->get_rate_id();
					if($this->settings['courier_shipping'] == 'yes'){
						$title = !empty($this->settings['courier_title']) ? $this->settings['courier_title'] : esc_html__('Courier Shipping', 'baikalsr-shipping');
						$price = (float)$calcResult["total"] + $this->settings['additional_cost'] + (float)$calcResult["total"] * $this->settings['additional_cost_percent'] / 100;
						//Если подходит по условиям к бесплатной доставке
						if ($this->settings['free_shipment'] && $total_price >= $this->baikalsrTools::intOrFloat($this->settings['free_shipment'])) {	
							$price = 0.0;
						}
						$deliveryDate = '';						
						if(isset($calcResult["transit"]["int"]) && isset($calcResult["transit"]["day"])){
							$deliveryDate = $calcResult["transit"]["int"] . ' ' . $calcResult["transit"]["day"];
						}
						// Register the rate
						$this->add_rate([
							'id' => $id.'_courier',
							'label' => $this->settings['show_term'] == 'yes' ? sprintf('%s, %s', $title, $deliveryDate) : $title,
							'cost' => $price,
							'calc_tax' => 'per_item',
							'meta_data' => [
								'_destination_guid' => $destination['guid'],
								'_destination_name' => $destination['name'],
								'_smethod' => 'courier',
							],
						]);				
					}
					if($this->settings['pickup_shipping'] == 'yes'){
						//Проверим есть ли терминалы в городе получения
						$terminals = [];				
						$affiliate_data = $this->baikalsrTools::_getAffiliateData($destination['guid']);
						if (isset($affiliate_data["terminals"]) && !empty($affiliate_data["terminals"])) {
							foreach($affiliate_data["terminals"] as $terminal) 
								if (isset($terminal['finish']) && $terminal['finish']) {
									$terminals[] = array(
										'id' => isset($terminal['id']) && !empty($terminal['id']) ? $terminal['id'] : 0,
										'code' => isset($terminal['code']) && !empty($terminal['code']) ? htmlentities($terminal['code']) : '',
										'name' => htmlentities($terminal['name']),
										'address' => htmlentities($terminal["address"]),
										'schedule' => array($terminal['schedule_1'], $terminal['schedule_2'], $terminal['schedule_3'], $terminal['schedule_4'], $terminal['schedule_5'], $terminal['schedule_6'], $terminal['schedule_7'])
									);
								}
						}
						//Если нет терминалов для выбора, значит возможна доставка только по адресу
						if(!empty($terminals)){
							$title = !empty($this->settings['pickup_title']) ? $this->settings['pickup_title'] : esc_html__('Pickup Shipping', 'baikalsr-shipping');
							$price = (float)$calcResult["total"] - $destination_total + $this->settings['additional_cost'] + ((float)$calcResult["total"] - $destination_total) * $this->settings['additional_cost_percent'] / 100;
							//Если подходит по условиям к бесплатной доставке
							if ($this->settings['free_shipment'] && $total_price >= $this->baikalsrTools::intOrFloat($this->settings['free_shipment'])) {	
								$price = 0.0;
							}
							$deliveryDate = '';		
							if(isset($calcResult["transit"]["int"]) && isset($calcResult["transit"]["day"])){
								if((int)$calcResult["transit"]["int"]>1)
									$calcResult["transit"]["int"] = (int)$calcResult["transit"]["int"] - 1;
								$deliveryDate = $calcResult["transit"]["int"] . ' ' . $calcResult["transit"]["day"];
							}
							// Register the rate
							$this->add_rate([
								'id' => $id.'_pickup',
								'label' => $this->settings['show_term'] == 'yes' ? sprintf('%s, %s', $title, $deliveryDate) : $title,
								'cost' => $price,
								'calc_tax' => 'per_item',
								'meta_data' => [
									'_destination_guid' => $destination['guid'],
									'_destination_name' => $destination['name'],
									'_smethod' => 'pickup',
									'_terminals' => $terminals,
									'_preferred_terminal' => $terminals[0],
								],
							]);
						}
					}			
				}
			}
		}
		
		public function shipping_rate_description( $method ) {
			if ( $method->id === 'baikalsr_shipping_pickup' ) {
				echo '<p>Some HTML in here where you can explain the benefits of choosing this shipping rate</p>';
			}
		}
		
		public function _getDimensionsAndWeight($items, $shipping_params) {
			$arDimensions = array();
			$defaultForItem = $shipping_params['default_for_item'];
			$defLength = wc_get_dimension($this->baikalsrTools::intOrFloat($shipping_params['lengthD']), 'm', $shipping_params['lwh_unit']);
			$defWidth = wc_get_dimension($this->baikalsrTools::intOrFloat($shipping_params['widthD']), 'm', $shipping_params['lwh_unit']);
			$defHeight = wc_get_dimension($this->baikalsrTools::intOrFloat($shipping_params['heightD']), 'm', $shipping_params['lwh_unit']);
			$defWeight = $this->baikalsrTools::intOrFloat(wc_get_weight($shipping_params['weightD'], 'kg', $shipping_params['weight_unit']));
			$totalVolume1 = 0;
			$totalWeight1 = 0;
			$totalVolume2 = 0;
			$totalWeight2 = 0;
			$maxLength = 0;
			$maxWidth = 0;
			$maxHeight = 0;
			//max dimension
			foreach ($items as $item) {
				if(isset($item['data']) && isset($item['quantity'])){
					$product = $item['data'];
					$quantity = $item['quantity'];
				}else{
					$product = $item->get_product();
					$quantity = $item->get_quantity();
				}
				$weight = $this->baikalsrTools::intOrFloat(wc_get_weight($product->get_weight(), 'kg'));
				if(!empty($weight)){
					$totalWeight1 += $quantity * $weight;	
				}else{
					if($defaultForItem)
						$totalWeight2 += $quantity * $defWeight;
					else
						$totalWeight2 = $defWeight;
				}	
				$length = wc_get_dimension($this->baikalsrTools::intOrFloat($product->get_length()), 'm');
				$width = wc_get_dimension($this->baikalsrTools::intOrFloat($product->get_width()), 'm');
				$height = wc_get_dimension($this->baikalsrTools::intOrFloat($product->get_height()), 'm');					
				if(!empty($length) && !empty($width) && !empty($height)){
					$totalVolume1 += $quantity * $length * $width * $height;
				}else{
					$length = $defLength;
					$width = $defWidth;
					$height = $defHeight;
					if($defaultForItem)
						$totalVolume2 += $quantity * $defLength * $defWidth * $defHeight;
					else
						$totalVolume2 = $defLength * $defWidth * $defHeight;
				}
				if($length > $maxLength) $maxLength = $length;
				if($width > $maxWidth) $maxWidth = $width;
				if($height > $maxHeight) $maxHeight = $height;
			}
			$arDimensions['length']= $maxLength;
			$arDimensions['width'] = $maxWidth;
			$arDimensions['height'] = $maxHeight;
			$arDimensions['volume'] = $totalVolume1 + $totalVolume2;
			$arDimensions['weight'] = $totalWeight1 + $totalWeight2;
			return $arDimensions;
		}	
		
		private static function logger(): \WC_Logger_Interface
		{
			return wc_get_logger();
		}
	}
}